@@ -35,6 +35,7 @@ import org.slf4j.LoggerFactory;
 
 import org.apache.uniffle.client.api.ShuffleWriteClient;
 import org.apache.uniffle.client.factory.ShuffleClientFactory;
+import org.apache.uniffle.client.util.ClientUtils;
 import org.apache.uniffle.common.ShuffleServerInfo;
 import org.apache.uniffle.common.exception.RssException;
 import org.apache.uniffle.common.util.BlockIdLayout;
@@ -44,37 +45,61 @@ public class RssMRUtils {
 
   private static final Logger LOG = LoggerFactory.getLogger(RssMRUtils.class);
   private static final BlockIdLayout LAYOUT = BlockIdLayout.DEFAULT;
-  private static final int MAX_ATTEMPT_LENGTH = 6;
-  private static final int MAX_ATTEMPT_ID = (1 << MAX_ATTEMPT_LENGTH) - 1;
-  private static final int MAX_SEQUENCE_NO =
-      (1 << (LAYOUT.sequenceNoBits - MAX_ATTEMPT_LENGTH)) - 1;
 
-  // Class TaskAttemptId have two field id and mapId, rss taskAttemptID have 21 bits,
-  // mapId is 19 bits, id is 2 bits. MR have a trick logic, taskAttemptId will increase
-  // 1000 * (appAttemptId - 1), so we will decrease it.
-  public static long convertTaskAttemptIdToLong(TaskAttemptID taskAttemptID, int appAttemptId) {
-    int lowBytes = taskAttemptID.getTaskID().getId();
-    if (lowBytes > LAYOUT.maxTaskAttemptId) {
-      throw new RssException("TaskAttempt " + taskAttemptID + " low bytes " + lowBytes + " exceed");
-    }
+  // Class TaskAttemptId have two field id and mapId. MR have a trick logic, taskAttemptId will
+  // increase 1000 * (appAttemptId - 1), so we will decrease it.
+  public static int createRssTaskAttemptId(
+      TaskAttemptID taskAttemptID, int appAttemptId, int maxAttemptNo) {
+    int attemptBits = ClientUtils.getNumberOfSignificantBits(maxAttemptNo);
+
     if (appAttemptId < 1) {
       throw new RssException("appAttemptId " + appAttemptId + " is wrong");
     }
-    int highBytes = taskAttemptID.getId() - (appAttemptId - 1) * 1000;
-    if (highBytes > MAX_ATTEMPT_ID || highBytes < 0) {
+    int attemptId = taskAttemptID.getId() - (appAttemptId - 1) * 1000;
+    if (attemptId > maxAttemptNo || attemptId < 0) {
       throw new RssException(
-          "TaskAttempt " + taskAttemptID + " high bytes " + highBytes + " exceed");
+          "TaskAttempt " + taskAttemptID + " attemptId " + attemptId + " exceed " + maxAttemptNo);
     }
-    return LAYOUT.getBlockId(highBytes, 0, lowBytes);
+    int taskId = taskAttemptID.getTaskID().getId();
+
+    int mapIndexBits = ClientUtils.getNumberOfSignificantBits(taskId);
+    if (mapIndexBits + attemptBits > LAYOUT.taskAttemptIdBits) {
+      throw new RssException(
+          "Observing taskId["
+              + taskId
+              + "] that would produce a taskAttemptId with "
+              + (mapIndexBits + attemptBits)
+              + " bits which is larger than the allowed "
+              + LAYOUT.taskAttemptIdBits
+              + "]). Please consider providing more bits for taskAttemptIds.");
+    }
+
+    return (taskId << attemptBits) | attemptId;
+  }
+
+  public static int createRssTaskAttemptId(
+      TaskAttemptID taskAttemptID, int appAttemptId, int maxFailures, boolean speculation) {
+    int maxAttemptNo = ClientUtils.getMaxAttemptNo(maxFailures, speculation);
+    return createRssTaskAttemptId(taskAttemptID, appAttemptId, maxAttemptNo);
+  }
+
+  public static int createRssTaskAttemptId(
+      TaskAttemptID taskAttemptID, int appAttemptId, Configuration conf) {
+    int maxFailures = conf.getInt(MRJobConfig.MAP_MAX_ATTEMPTS, 4);
+    boolean speculation = conf.getBoolean(MRJobConfig.MAP_SPECULATIVE, true);
+    return createRssTaskAttemptId(taskAttemptID, appAttemptId, maxFailures, speculation);
   }
 
   public static TaskAttemptID createMRTaskAttemptId(
-      JobID jobID, TaskType taskType, long rssTaskAttemptId, int appAttemptId) {
+      JobID jobID, TaskType taskType, long rssTaskAttemptId, int appAttemptId, int maxAttemptNo) {
+    int attemptBits = ClientUtils.getNumberOfSignificantBits(maxAttemptNo);
     if (appAttemptId < 1) {
       throw new RssException("appAttemptId " + appAttemptId + " is wrong");
     }
-    TaskID taskID = new TaskID(jobID, taskType, LAYOUT.getTaskAttemptId(rssTaskAttemptId));
-    int id = LAYOUT.getSequenceNo(rssTaskAttemptId) + 1000 * (appAttemptId - 1);
+    int task = (int) rssTaskAttemptId >> attemptBits;
+    int attempt = (int) rssTaskAttemptId & ((1 << attemptBits) - 1);
+    TaskID taskID = new TaskID(jobID, taskType, task);
+    int id = attempt + 1000 * (appAttemptId - 1);
     return new TaskAttemptID(taskID, id);
   }
 
@@ -228,27 +253,11 @@ public class RssMRUtils {
   }
 
   public static long getBlockId(int partitionId, long taskAttemptId, int nextSeqNo) {
-    long attemptId = taskAttemptId >> (LAYOUT.partitionIdBits + LAYOUT.taskAttemptIdBits);
-    if (attemptId < 0 || attemptId > MAX_ATTEMPT_ID) {
-      throw new RssException(
-          "Can't support attemptId [" + attemptId + "], the max value should be " + MAX_ATTEMPT_ID);
-    }
-    if (nextSeqNo < 0 || nextSeqNo > MAX_SEQUENCE_NO) {
-      throw new RssException(
-          "Can't support sequence [" + nextSeqNo + "], the max value should be " + MAX_SEQUENCE_NO);
-    }
-
-    int atomicInt = (int) ((nextSeqNo << MAX_ATTEMPT_LENGTH) + attemptId);
-    long taskId =
-        taskAttemptId - (attemptId << (LAYOUT.partitionIdBits + LAYOUT.taskAttemptIdBits));
-
-    return LAYOUT.getBlockId(atomicInt, partitionId, taskId);
+    return LAYOUT.getBlockId(nextSeqNo, partitionId, taskAttemptId);
   }
 
-  public static long getTaskAttemptId(long blockId) {
-    int mapId = LAYOUT.getTaskAttemptId(blockId);
-    int attemptId = LAYOUT.getSequenceNo(blockId) & MAX_ATTEMPT_ID;
-    return LAYOUT.getBlockId(attemptId, 0, mapId);
+  public static int getTaskAttemptId(long blockId) {
+    return LAYOUT.getTaskAttemptId(blockId);
   }
 
   public static int estimateTaskConcurrency(JobConf jobConf) {
