@@ -18,6 +18,7 @@
 package org.apache.spark.shuffle.reader;
 
 import java.util.List;
+import java.util.Map;
 
 import scala.Function0;
 import scala.Option;
@@ -45,7 +46,6 @@ import org.slf4j.LoggerFactory;
 
 import org.apache.uniffle.client.api.ShuffleReadClient;
 import org.apache.uniffle.client.factory.ShuffleClientFactory;
-import org.apache.uniffle.client.request.CreateShuffleReadClientRequest;
 import org.apache.uniffle.client.util.RssClientConfig;
 import org.apache.uniffle.common.ShuffleServerInfo;
 import org.apache.uniffle.common.config.RssClientConf;
@@ -84,7 +84,8 @@ public class RssShuffleReader<K, C> implements ShuffleReader<K, C> {
       int partitionNum,
       Roaring64NavigableMap blockIdBitmap,
       Roaring64NavigableMap taskIdBitmap,
-      RssConf rssConf) {
+      RssConf rssConf,
+      Map<Integer, List<ShuffleServerInfo>> partitionToServers) {
     this.appId = rssShuffleHandle.getAppId();
     this.startPartition = startPartition;
     this.endPartition = endPartition;
@@ -99,8 +100,7 @@ public class RssShuffleReader<K, C> implements ShuffleReader<K, C> {
     this.blockIdBitmap = blockIdBitmap;
     this.taskIdBitmap = taskIdBitmap;
     this.hadoopConf = hadoopConf;
-    this.shuffleServerInfoList =
-        (List<ShuffleServerInfo>) (rssShuffleHandle.getPartitionToServers().get(startPartition));
+    this.shuffleServerInfoList = (List<ShuffleServerInfo>) (partitionToServers.get(startPartition));
     this.rssConf = rssConf;
     expectedTaskIdsBitmapFilterEnable = shuffleServerInfoList.size() > 1;
   }
@@ -108,23 +108,32 @@ public class RssShuffleReader<K, C> implements ShuffleReader<K, C> {
   @Override
   public Iterator<Product2<K, C>> read() {
     LOG.info("Shuffle read started:" + getReadInfo());
-
-    CreateShuffleReadClientRequest request =
-        new CreateShuffleReadClientRequest(
-            appId,
-            shuffleId,
-            startPartition,
-            basePath,
-            partitionNumPerRange,
-            partitionNum,
-            blockIdBitmap,
-            taskIdBitmap,
-            shuffleServerInfoList,
-            hadoopConf,
-            expectedTaskIdsBitmapFilterEnable,
-            rssConf);
+    int retryMax =
+        rssConf.getInteger(
+            RssClientConfig.RSS_CLIENT_RETRY_MAX,
+            RssClientConfig.RSS_CLIENT_RETRY_MAX_DEFAULT_VALUE);
+    long retryIntervalMax =
+        rssConf.getLong(
+            RssClientConfig.RSS_CLIENT_RETRY_INTERVAL_MAX,
+            RssClientConfig.RSS_CLIENT_RETRY_INTERVAL_MAX_DEFAULT_VALUE);
     ShuffleReadClient shuffleReadClient =
-        ShuffleClientFactory.getInstance().createShuffleReadClient(request);
+        ShuffleClientFactory.getInstance()
+            .createShuffleReadClient(
+                ShuffleClientFactory.newReadBuilder()
+                    .appId(appId)
+                    .shuffleId(shuffleId)
+                    .partitionId(startPartition)
+                    .basePath(basePath)
+                    .partitionNumPerRange(partitionNumPerRange)
+                    .partitionNum(partitionNum)
+                    .blockIdBitmap(blockIdBitmap)
+                    .taskIdBitmap(taskIdBitmap)
+                    .shuffleServerInfoList(shuffleServerInfoList)
+                    .hadoopConf(hadoopConf)
+                    .expectedTaskIdsBitmapFilterEnable(expectedTaskIdsBitmapFilterEnable)
+                    .retryMax(retryMax)
+                    .retryIntervalMax(retryIntervalMax)
+                    .rssConf(rssConf));
     RssShuffleDataIterator rssShuffleDataIterator =
         new RssShuffleDataIterator<K, C>(
             shuffleDependency.serializer(),
