@@ -19,11 +19,14 @@ package org.apache.spark.shuffle;
 
 import org.junit.jupiter.api.Test;
 
+import static org.junit.jupiter.api.Assertions.assertFalse;
 import static org.junit.jupiter.api.Assertions.assertTrue;
 
 public class SparkVersionUtilsTest {
   @Test
   public void testSparkVersion() {
     assertTrue(SparkVersionUtils.isSpark2());
+    assertFalse(SparkVersionUtils.isSpark3());
+    assertFalse(SparkVersionUtils.isSpark320());
   }
 }
