@@ -28,6 +28,7 @@ import java.util.Set;
 import java.util.function.Supplier;
 import java.util.stream.Collectors;
 
+import com.google.common.collect.Lists;
 import com.google.common.collect.Sets;
 import org.apache.hadoop.conf.Configuration;
 import org.apache.hadoop.fs.FileSystem;
@@ -54,6 +55,7 @@ import org.junit.jupiter.api.io.TempDir;
 import org.roaringbitmap.longlong.Roaring64NavigableMap;
 
 import org.apache.uniffle.client.api.ShuffleWriteClient;
+import org.apache.uniffle.client.impl.FailedBlockSendTracker;
 import org.apache.uniffle.client.response.SendShuffleDataResult;
 import org.apache.uniffle.common.PartitionRange;
 import org.apache.uniffle.common.RemoteStorageInfo;
@@ -63,11 +65,13 @@ import org.apache.uniffle.common.ShuffleDataDistributionType;
 import org.apache.uniffle.common.ShuffleServerInfo;
 import org.apache.uniffle.common.config.RssConf;
 import org.apache.uniffle.common.exception.RssException;
+import org.apache.uniffle.common.rpc.StatusCode;
 import org.apache.uniffle.storage.util.StorageType;
 
 import static org.junit.jupiter.api.Assertions.assertEquals;
 import static org.junit.jupiter.api.Assertions.assertThrows;
 import static org.junit.jupiter.api.Assertions.assertTrue;
+import static org.mockito.Mockito.mock;
 
 public class WriteBufferManagerTest {
   @Test
@@ -151,7 +155,7 @@ public class WriteBufferManagerTest {
             true,
             mapOutputByteCounter,
             mapOutputRecordCounter);
-
+    partitionToServers.put(1, Lists.newArrayList(mock(ShuffleServerInfo.class)));
     Random random = new Random();
     for (int i = 0; i < 1000; i++) {
       byte[] key = new byte[20];
@@ -263,6 +267,7 @@ public class WriteBufferManagerTest {
       random.nextBytes(key);
       random.nextBytes(value);
       int partitionId = random.nextInt(50);
+      partitionToServers.put(partitionId, Lists.newArrayList(mock(ShuffleServerInfo.class)));
       bufferManager.addRecord(partitionId, new BytesWritable(key), new BytesWritable(value));
     }
 
@@ -377,6 +382,7 @@ public class WriteBufferManagerTest {
       random.nextBytes(key);
       random.nextBytes(value);
       int partitionId = random.nextInt(50);
+      partitionToServers.put(partitionId, Lists.newArrayList(mock(ShuffleServerInfo.class)));
       bufferManager.addRecord(partitionId, new BytesWritable(key), new BytesWritable(value));
     }
     bufferManager.waitSendFinished();
@@ -484,6 +490,8 @@ public class WriteBufferManagerTest {
                 random.nextBytes(key);
                 random.nextBytes(value);
                 int partitionId = random.nextInt(50);
+                partitionToServers.put(
+                    partitionId, Lists.newArrayList(mock(ShuffleServerInfo.class)));
                 bufferManager.addRecord(
                     partitionId, new BytesWritable(key), new BytesWritable(value));
               }
@@ -541,7 +549,12 @@ public class WriteBufferManagerTest {
       if (mode == 0) {
         throw new RssException("send data failed.");
       } else if (mode == 1) {
-        return new SendShuffleDataResult(Sets.newHashSet(2L), Sets.newHashSet(1L));
+        FailedBlockSendTracker failedBlockSendTracker = new FailedBlockSendTracker();
+        ShuffleBlockInfo failedBlock =
+            new ShuffleBlockInfo(1, 1, 3, 1, 1, new byte[1], null, 1, 100, 1);
+        failedBlockSendTracker.add(
+            failedBlock, new ShuffleServerInfo("host", 39998), StatusCode.NO_BUFFER);
+        return new SendShuffleDataResult(Sets.newHashSet(2L), failedBlockSendTracker);
       } else {
         if (mode == 3) {
           try {
@@ -556,7 +569,7 @@ public class WriteBufferManagerTest {
         for (ShuffleBlockInfo blockInfo : shuffleBlockInfoList) {
           successBlockIds.add(blockInfo.getBlockId());
         }
-        return new SendShuffleDataResult(successBlockIds, Sets.newHashSet());
+        return new SendShuffleDataResult(successBlockIds, new FailedBlockSendTracker());
       }
     }
 
@@ -604,17 +617,21 @@ public class WriteBufferManagerTest {
 
     @Override
     public void reportShuffleResult(
-        Map<Integer, List<ShuffleServerInfo>> partitionToServers,
+        Map<ShuffleServerInfo, Map<Integer, Set<Long>>> serverToPartitionToBlockIds,
         String appId,
         int shuffleId,
         long taskAttemptId,
-        Map<Integer, List<Long>> partitionToBlockIds,
         int bitmapNum) {
       if (mode == 3) {
-        mockedShuffleServer.addFinishedBlockInfos(
-            partitionToBlockIds.values().stream()
-                .flatMap(it -> it.stream())
-                .collect(Collectors.toList()));
+        serverToPartitionToBlockIds
+            .values()
+            .forEach(
+                partitionToBlockIds -> {
+                  mockedShuffleServer.addFinishedBlockInfos(
+                      partitionToBlockIds.values().stream()
+                          .flatMap(it -> it.stream())
+                          .collect(Collectors.toList()));
+                });
       }
     }
 
