@@ -17,12 +17,18 @@
 
 package org.apache.uniffle.client.util;
 
-import org.apache.uniffle.common.util.Constants;
+import org.apache.uniffle.common.util.BlockIdLayout;
 import org.apache.uniffle.common.util.IdHelper;
 
 public class DefaultIdHelper implements IdHelper {
+  private final BlockIdLayout layout;
+
+  public DefaultIdHelper(BlockIdLayout layout) {
+    this.layout = layout;
+  }
+
   @Override
   public long getTaskAttemptId(long blockId) {
-    return blockId & Constants.MAX_TASK_ATTEMPT_ID;
+    return layout.getTaskAttemptId(blockId);
   }
 }
