@@ -15,7 +15,6 @@
  * limitations under the License.
  */
 
-
 package org.apache.uniffle.common.exception;
 
 public class RssException extends RuntimeException {
@@ -24,6 +23,10 @@ public class RssException extends RuntimeException {
     super(message);
   }
 
+  public RssException(Throwable e) {
+    super(e);
+  }
+
   public RssException(String message, Throwable e) {
     super(message, e);
   }
