@@ -47,19 +47,11 @@ public class ShuffleServerInfoTest {
   public void testToString() {
     ShuffleServerInfo info = new ShuffleServerInfo("1", "localhost", 1234);
     assertEquals(
-        "ShuffleServerInfo{id["
-            + info.getId()
-            + "], host["
-            + info.getHost()
-            + "], grpc port["
-            + info.getGrpcPort()
-            + "]}",
+        "ShuffleServerInfo{host[" + info.getHost() + "], grpc port[" + info.getGrpcPort() + "]}",
         info.toString());
     ShuffleServerInfo newInfo = new ShuffleServerInfo("1", "localhost", 1234, 5678);
     assertEquals(
-        "ShuffleServerInfo{id["
-            + info.getId()
-            + "], host["
+        "ShuffleServerInfo{host["
             + newInfo.getHost()
             + "], grpc port["
             + newInfo.getGrpcPort()
