@@ -49,7 +49,8 @@ public class AccessManager implements Reconfigurable {
       CoordinatorConf conf,
       ClusterManager clusterManager,
       QuotaManager quotaManager,
-      Configuration hadoopConf) throws Exception {
+      Configuration hadoopConf)
+      throws Exception {
     this.coordinatorConf = conf;
     this.clusterManager = clusterManager;
     this.hadoopConf = hadoopConf;
