@@ -20,6 +20,7 @@ package org.apache.uniffle.coordinator;
 import java.io.Closeable;
 import java.net.URI;
 import java.net.URISyntaxException;
+import java.nio.charset.StandardCharsets;
 import java.util.ArrayList;
 import java.util.Arrays;
 import java.util.Collections;
@@ -34,6 +35,8 @@ import com.google.common.annotations.VisibleForTesting;
 import com.google.common.collect.Lists;
 import com.google.common.collect.Maps;
 import com.google.common.collect.Sets;
+import org.apache.commons.collections.MapUtils;
+import org.apache.commons.io.IOUtils;
 import org.apache.commons.lang3.Range;
 import org.apache.commons.lang3.StringUtils;
 import org.slf4j.Logger;
@@ -41,16 +44,16 @@ import org.slf4j.LoggerFactory;
 
 import org.apache.uniffle.common.Application;
 import org.apache.uniffle.common.RemoteStorageInfo;
-import org.apache.uniffle.common.util.Constants;
 import org.apache.uniffle.common.util.JavaUtils;
 import org.apache.uniffle.common.util.ThreadUtils;
 import org.apache.uniffle.coordinator.access.checker.AccessQuotaChecker;
+import org.apache.uniffle.coordinator.conf.ClientConf;
+import org.apache.uniffle.coordinator.conf.LegacyClientConfParser;
 import org.apache.uniffle.coordinator.metric.CoordinatorMetrics;
 import org.apache.uniffle.coordinator.strategy.storage.AppBalanceSelectStorageStrategy;
 import org.apache.uniffle.coordinator.strategy.storage.LowestIOSampleCostSelectStorageStrategy;
 import org.apache.uniffle.coordinator.strategy.storage.RankValue;
 import org.apache.uniffle.coordinator.strategy.storage.SelectStorageStrategy;
-import org.apache.uniffle.coordinator.util.CoordinatorUtils;
 
 public class ApplicationManager implements Closeable {
 
@@ -152,48 +155,69 @@ public class ApplicationManager implements Closeable {
     }
   }
 
-  public void refreshRemoteStorage(String remoteStoragePath, String remoteStorageConf) {
-    if (!StringUtils.isEmpty(remoteStoragePath)) {
-      LOG.info("Refresh remote storage with {} {}", remoteStoragePath, remoteStorageConf);
-      Set<String> paths = Sets.newHashSet(remoteStoragePath.split(Constants.COMMA_SPLIT_CHAR));
-      Map<String, Map<String, String>> confKVs =
-          CoordinatorUtils.extractRemoteStorageConf(remoteStorageConf);
-      // add remote path if not exist
-      for (String path : paths) {
-        if (!availableRemoteStorageInfo.containsKey(path)) {
-          remoteStoragePathRankValue.computeIfAbsent(
-              path,
-              key -> {
-                // refreshRemoteStorage is designed without multiple thread problem
-                // metrics shouldn't be added duplicated
-                addRemoteStorageMetrics(path);
-                return new RankValue(0);
-              });
-        }
-        String storageHost = getStorageHost(path);
-        RemoteStorageInfo rsInfo =
-            new RemoteStorageInfo(path, confKVs.getOrDefault(storageHost, Maps.newHashMap()));
-        availableRemoteStorageInfo.put(path, rsInfo);
-      }
-      // remove unused remote path if exist
-      List<String> unusedPath = Lists.newArrayList();
-      for (String existPath : availableRemoteStorageInfo.keySet()) {
-        if (!paths.contains(existPath)) {
-          unusedPath.add(existPath);
-        }
-      }
-      // remote unused path
-      for (String path : unusedPath) {
-        availableRemoteStorageInfo.remove(path);
-        // try to remove if counter = 0, or it will be removed in decRemoteStorageCounter() later
-        removePathFromCounter(path);
-      }
-    } else {
-      LOG.info("Refresh remote storage with empty value {}", remoteStoragePath);
+  public void refreshRemoteStorages(ClientConf dynamicClientConf) {
+    if (dynamicClientConf == null || MapUtils.isEmpty(dynamicClientConf.getRemoteStorageInfos())) {
+      LOG.info("Refresh remote storage with empty config");
       for (String path : availableRemoteStorageInfo.keySet()) {
         removePathFromCounter(path);
       }
       availableRemoteStorageInfo.clear();
+      return;
+    }
+
+    Map<String, RemoteStorageInfo> remoteStorageInfoMap = dynamicClientConf.getRemoteStorageInfos();
+    LOG.info("Refresh remote storage with {}", remoteStorageInfoMap);
+
+    for (Map.Entry<String, RemoteStorageInfo> entry : remoteStorageInfoMap.entrySet()) {
+      String path = entry.getKey();
+      RemoteStorageInfo rsInfo = entry.getValue();
+
+      if (!availableRemoteStorageInfo.containsKey(path)) {
+        remoteStoragePathRankValue.computeIfAbsent(
+            path,
+            key -> {
+              // refreshRemoteStorage is designed without multiple thread problem
+              // metrics shouldn't be added duplicated
+              addRemoteStorageMetrics(path);
+              return new RankValue(0);
+            });
+      }
+      availableRemoteStorageInfo.put(path, rsInfo);
+    }
+    // remove unused remote path if exist
+    List<String> unusedPath = Lists.newArrayList();
+    for (String existPath : availableRemoteStorageInfo.keySet()) {
+      if (!remoteStorageInfoMap.containsKey(existPath)) {
+        unusedPath.add(existPath);
+      }
+    }
+    // remote unused path
+    for (String path : unusedPath) {
+      availableRemoteStorageInfo.remove(path);
+      // try to remove if counter = 0, or it will be removed in decRemoteStorageCounter() later
+      removePathFromCounter(path);
+    }
+  }
+
+  // only for test
+  @VisibleForTesting
+  public void refreshRemoteStorage(String remoteStoragePath, String remoteStorageConf) {
+    try {
+      LegacyClientConfParser parser = new LegacyClientConfParser();
+
+      String remoteStorageConfRaw =
+          String.format(
+              "%s %s %n %s %s",
+              CoordinatorConf.COORDINATOR_REMOTE_STORAGE_PATH.key(),
+              remoteStoragePath,
+              CoordinatorConf.COORDINATOR_REMOTE_STORAGE_CLUSTER_CONF.key(),
+              remoteStorageConf);
+
+      ClientConf conf =
+          parser.tryParse(IOUtils.toInputStream(remoteStorageConfRaw, StandardCharsets.UTF_8));
+      refreshRemoteStorages(conf);
+    } catch (Exception e) {
+      LOG.error("Errors on refreshing remote storage", e);
     }
   }
 
@@ -503,6 +527,10 @@ public class ApplicationManager implements Closeable {
     return REMOTE_PATH_SCHEMA;
   }
 
+  public Map<String, Map<String, Long>> getCurrentUserAndApp() {
+    return currentUserAndApp;
+  }
+
   public void close() {
     if (detectStorageScheduler != null) {
       detectStorageScheduler.shutdownNow();
