@@ -23,6 +23,7 @@ import org.apache.uniffle.common.config.ConfigOption;
 import org.apache.uniffle.common.config.ConfigOptions;
 import org.apache.uniffle.common.config.ConfigUtils;
 import org.apache.uniffle.common.config.RssBaseConf;
+import org.apache.uniffle.coordinator.conf.ClientConfParser;
 import org.apache.uniffle.coordinator.strategy.assignment.AbstractAssignmentStrategy;
 import org.apache.uniffle.coordinator.strategy.assignment.AssignmentStrategyFactory;
 
@@ -129,6 +130,12 @@ public class CoordinatorConf extends RssBaseConf {
           .stringType()
           .noDefaultValue()
           .withDescription("dynamic client conf of this cluster");
+  public static final ConfigOption<ClientConfParser.Parser>
+      COORDINATOR_DYNAMIC_CLIENT_RAW_CONF_PARSER =
+          ConfigOptions.key("rss.coordinator.dynamicClientConf.parser")
+              .enumType(ClientConfParser.Parser.class)
+              .defaultValue(ClientConfParser.Parser.LEGACY)
+              .withDescription("dynamic client conf parser");
   public static final ConfigOption<String> COORDINATOR_REMOTE_STORAGE_PATH =
       ConfigOptions.key("rss.coordinator.remote.storage.path")
           .stringType()
@@ -201,13 +208,17 @@ public class CoordinatorConf extends RssBaseConf {
   public static final ConfigOption<Integer> COORDINATOR_QUOTA_DEFAULT_APP_NUM =
       ConfigOptions.key("rss.coordinator.quota.default.app.num")
           .intType()
-          .defaultValue(5)
-          .withDescription("Default number of apps at user level");
+          .defaultValue(-1)
+          .withDescription(
+              "Default number of apps at user level, if set value < 0,then this user app quota "
+                  + "check would passed");
   public static final ConfigOption<String> COORDINATOR_QUOTA_DEFAULT_PATH =
       ConfigOptions.key("rss.coordinator.quota.default.path")
           .stringType()
           .noDefaultValue()
-          .withDescription("A configuration file for the number of apps for a user-defined user");
+          .withDescription(
+              "A configuration file for the number of apps for a user-defined user, if set value < 0,then this user app quota "
+                  + "check would passed.");
   public static final ConfigOption<Long> COORDINATOR_QUOTA_UPDATE_INTERVAL =
       ConfigOptions.key("rss.coordinator.quota.update.interval")
           .longType()
