@@ -17,4 +17,7 @@
 # limitations under the License.
 #
 
-echo "Build kubernetes operator..."
+echo "Build and Test kubernetes operator..."
+
+# todo: separate the build process from test process, we need to modify the Makefile
+cd operator && make build docker-build
