@@ -22,8 +22,12 @@ import java.io.FileWriter;
 import java.util.ArrayList;
 import java.util.Arrays;
 import java.util.HashSet;
+import java.util.List;
+import java.util.Set;
 import java.util.concurrent.TimeUnit;
+import java.util.stream.Collectors;
 
+import com.google.common.collect.Lists;
 import com.google.common.collect.Sets;
 import com.google.common.util.concurrent.Uninterruptibles;
 import org.junit.jupiter.api.BeforeAll;
@@ -32,13 +36,17 @@ import org.junit.jupiter.api.io.TempDir;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 
+import org.apache.uniffle.client.factory.ShuffleClientFactory;
 import org.apache.uniffle.client.impl.ShuffleWriteClientImpl;
 import org.apache.uniffle.common.ClientType;
 import org.apache.uniffle.common.ShuffleAssignmentsInfo;
+import org.apache.uniffle.common.ShuffleServerInfo;
 import org.apache.uniffle.common.config.ReconfigurableBase;
 import org.apache.uniffle.common.config.RssBaseConf;
+import org.apache.uniffle.common.rpc.ServerType;
 import org.apache.uniffle.coordinator.CoordinatorConf;
 import org.apache.uniffle.coordinator.SimpleClusterManager;
+import org.apache.uniffle.server.ShuffleServer;
 import org.apache.uniffle.server.ShuffleServerConf;
 import org.apache.uniffle.storage.util.StorageType;
 
@@ -76,9 +84,9 @@ public class CoordinatorAssignmentTest extends CoordinatorTestBase {
     createCoordinatorServer(coordinatorConf2);
 
     for (int i = 0; i < SERVER_NUM; i++) {
-      ShuffleServerConf shuffleServerConf = getShuffleServerConf();
-      File dataDir1 = new File(tmpDir, "data1");
-      String basePath = dataDir1.getAbsolutePath();
+      ShuffleServerConf shuffleServerConf = getShuffleServerConf(ServerType.GRPC);
+      File dataDir = new File(tmpDir, "data" + i);
+      String basePath = dataDir.getAbsolutePath();
       shuffleServerConf.setString(
           ShuffleServerConf.RSS_STORAGE_TYPE.key(), StorageType.MEMORY_LOCALFILE_HDFS.name());
       shuffleServerConf.set(ShuffleServerConf.RSS_STORAGE_BASE_PATH, Arrays.asList(basePath));
@@ -99,7 +107,20 @@ public class CoordinatorAssignmentTest extends CoordinatorTestBase {
   @Test
   public void testSilentPeriod() throws Exception {
     ShuffleWriteClientImpl shuffleWriteClient =
-        new ShuffleWriteClientImpl(ClientType.GRPC.name(), 3, 1000, 1, 1, 1, 1, true, 1, 1, 10, 10);
+        ShuffleClientFactory.newWriteBuilder()
+            .clientType(ClientType.GRPC.name())
+            .retryMax(3)
+            .retryIntervalMax(1000)
+            .heartBeatThreadNum(1)
+            .replica(1)
+            .replicaWrite(1)
+            .replicaRead(1)
+            .replicaSkipEnabled(true)
+            .dataTransferPoolSize(1)
+            .dataCommitPoolSize(1)
+            .unregisterThreadPoolSize(10)
+            .unregisterRequestTimeSec(10)
+            .build();
     shuffleWriteClient.registerCoordinators(QUORUM);
 
     // Case1: Disable silent period
@@ -108,25 +129,39 @@ public class CoordinatorAssignmentTest extends CoordinatorTestBase {
     assertEquals(SHUFFLE_NODES_MAX, info.getServerToPartitionRanges().keySet().size());
 
     // Case2: Enable silent period mechanism, it should fallback to slave coordinator.
-    SimpleClusterManager clusterManager =
-        (SimpleClusterManager) coordinators.get(0).getClusterManager();
-    clusterManager.setReadyForServe(false);
-    clusterManager.setStartupSilentPeriodEnabled(true);
-    clusterManager.setStartTime(System.currentTimeMillis() - 1);
+    try (SimpleClusterManager clusterManager =
+        (SimpleClusterManager) coordinators.get(0).getClusterManager()) {
+      clusterManager.setReadyForServe(false);
+      clusterManager.setStartupSilentPeriodEnabled(true);
+      clusterManager.setStartTime(System.currentTimeMillis() - 1);
 
-    if (clusterManager.getNodesNum() < 10) {
-      info = shuffleWriteClient.getShuffleAssignments("app1", 0, 10, 1, TAGS, -1, -1);
-      assertEquals(SHUFFLE_NODES_MAX, info.getServerToPartitionRanges().keySet().size());
+      if (clusterManager.getNodesNum() < 10) {
+        info = shuffleWriteClient.getShuffleAssignments("app1", 0, 10, 1, TAGS, -1, -1);
+        assertEquals(SHUFFLE_NODES_MAX, info.getServerToPartitionRanges().keySet().size());
+      }
+
+      // recover
+      clusterManager.setReadyForServe(true);
     }
-
-    // recover
-    clusterManager.setReadyForServe(true);
   }
 
   @Test
   public void testAssignmentServerNodesNumber() throws Exception {
     ShuffleWriteClientImpl shuffleWriteClient =
-        new ShuffleWriteClientImpl(ClientType.GRPC.name(), 3, 1000, 1, 1, 1, 1, true, 1, 1, 10, 10);
+        ShuffleClientFactory.newWriteBuilder()
+            .clientType(ClientType.GRPC.name())
+            .retryMax(3)
+            .retryIntervalMax(1000)
+            .heartBeatThreadNum(1)
+            .replica(1)
+            .replicaWrite(1)
+            .replicaRead(1)
+            .replicaSkipEnabled(true)
+            .dataTransferPoolSize(1)
+            .dataCommitPoolSize(1)
+            .unregisterThreadPoolSize(10)
+            .unregisterRequestTimeSec(10)
+            .build();
     shuffleWriteClient.registerCoordinators(COORDINATOR_QUORUM);
 
     /**
@@ -168,7 +203,20 @@ public class CoordinatorAssignmentTest extends CoordinatorTestBase {
             .getString(ReconfigurableBase.RECONFIGURABLE_FILE_NAME, "");
     new File(fileName).createNewFile();
     ShuffleWriteClientImpl shuffleWriteClient =
-        new ShuffleWriteClientImpl(ClientType.GRPC.name(), 3, 1000, 1, 1, 1, 1, true, 1, 1, 10, 10);
+        ShuffleClientFactory.newWriteBuilder()
+            .clientType(ClientType.GRPC.name())
+            .retryMax(3)
+            .retryIntervalMax(1000)
+            .heartBeatThreadNum(1)
+            .replica(1)
+            .replicaWrite(1)
+            .replicaRead(1)
+            .replicaSkipEnabled(true)
+            .dataTransferPoolSize(1)
+            .dataCommitPoolSize(1)
+            .unregisterThreadPoolSize(10)
+            .unregisterRequestTimeSec(10)
+            .build();
     shuffleWriteClient.registerCoordinators(COORDINATOR_QUORUM);
     ShuffleAssignmentsInfo info =
         shuffleWriteClient.getShuffleAssignments("app1", 0, 10, 1, TAGS, SERVER_NUM + 10, -1);
@@ -185,4 +233,40 @@ public class CoordinatorAssignmentTest extends CoordinatorTestBase {
     }
     Uninterruptibles.sleepUninterruptibly(2, TimeUnit.SECONDS);
   }
+
+  @Test
+  public void testGetReShuffleAssignments() {
+    ShuffleWriteClientImpl shuffleWriteClient =
+        ShuffleClientFactory.newWriteBuilder()
+            .clientType(ClientType.GRPC.name())
+            .retryMax(3)
+            .retryIntervalMax(1000)
+            .heartBeatThreadNum(1)
+            .replica(1)
+            .replicaWrite(1)
+            .replicaRead(1)
+            .replicaSkipEnabled(true)
+            .dataTransferPoolSize(1)
+            .dataCommitPoolSize(1)
+            .unregisterThreadPoolSize(10)
+            .unregisterRequestTimeSec(10)
+            .build();
+    shuffleWriteClient.registerCoordinators(COORDINATOR_QUORUM);
+    Set<String> excludeServer = Sets.newConcurrentHashSet();
+    List<ShuffleServer> excludeShuffleServer =
+        grpcShuffleServers.stream().limit(3).collect(Collectors.toList());
+    excludeShuffleServer.stream().map(ss -> ss.getId()).peek(excludeServer::add);
+    ShuffleAssignmentsInfo shuffleAssignmentsInfo =
+        shuffleWriteClient.getShuffleAssignments(
+            "app1", 0, 10, 1, TAGS, SERVER_NUM + 10, -1, excludeServer);
+    List<ShuffleServerInfo> resultShuffle = Lists.newArrayList();
+    for (List<ShuffleServerInfo> ssis : shuffleAssignmentsInfo.getPartitionToServers().values()) {
+      resultShuffle.addAll(ssis);
+    }
+
+    List<String> resultShuffleServerId =
+        resultShuffle.stream().map(a -> a.getId()).collect(Collectors.toList());
+    assertEquals(true, resultShuffleServerId.retainAll(excludeServer));
+    assertEquals(true, resultShuffleServerId.isEmpty());
+  }
 }
