@@ -19,16 +19,20 @@ package org.apache.uniffle.client.request;
 
 import java.util.Set;
 
+import org.apache.uniffle.common.util.BlockIdLayout;
+
 public class RssGetShuffleResultForMultiPartRequest {
   private String appId;
   private int shuffleId;
   private Set<Integer> partitions;
+  private BlockIdLayout blockIdLayout;
 
   public RssGetShuffleResultForMultiPartRequest(
-      String appId, int shuffleId, Set<Integer> partitions) {
+      String appId, int shuffleId, Set<Integer> partitions, BlockIdLayout blockIdLayout) {
     this.appId = appId;
     this.shuffleId = shuffleId;
     this.partitions = partitions;
+    this.blockIdLayout = blockIdLayout;
   }
 
   public String getAppId() {
@@ -42,4 +46,8 @@ public class RssGetShuffleResultForMultiPartRequest {
   public Set<Integer> getPartitions() {
     return partitions;
   }
+
+  public BlockIdLayout getBlockIdLayout() {
+    return blockIdLayout;
+  }
 }
