@@ -17,15 +17,14 @@
 
 package org.apache.uniffle.client.response;
 
-import java.nio.ByteBuffer;
-
 import org.apache.uniffle.common.ShuffleIndexResult;
+import org.apache.uniffle.common.netty.buffer.ManagedBuffer;
 import org.apache.uniffle.common.rpc.StatusCode;
 
 public class RssGetShuffleIndexResponse extends ClientResponse {
   private final ShuffleIndexResult shuffleIndexResult;
 
-  public RssGetShuffleIndexResponse(StatusCode statusCode, ByteBuffer data, long dataFileLen) {
+  public RssGetShuffleIndexResponse(StatusCode statusCode, ManagedBuffer data, long dataFileLen) {
     super(statusCode);
     this.shuffleIndexResult = new ShuffleIndexResult(data, dataFileLen);
   }
