@@ -24,20 +24,16 @@ import org.apache.uniffle.storage.handler.api.ShuffleWriteHandler;
 import org.apache.uniffle.storage.request.CreateShuffleReadHandlerRequest;
 import org.apache.uniffle.storage.request.CreateShuffleWriteHandlerRequest;
 
-
 public interface Storage {
 
   boolean canWrite();
 
-  boolean lockShuffleShared(String shuffleKey);
-
-  boolean unlockShuffleShared(String shuffleKey);
-
   void updateWriteMetrics(StorageWriteMetrics metrics);
 
   void updateReadMetrics(StorageReadMetrics metrics);
 
-  ShuffleWriteHandler getOrCreateWriteHandler(CreateShuffleWriteHandlerRequest request) throws IOException;
+  ShuffleWriteHandler getOrCreateWriteHandler(CreateShuffleWriteHandlerRequest request)
+      throws IOException;
 
   ServerReadHandler getOrCreateReadHandler(CreateShuffleReadHandlerRequest request);
 
